display.setStatusBar(display.HiddenStatusBar)
local sprite = require("sprite")

--these 2 variables will be the checks that control our event system.
local inEvent = 0
local eventRun = 0

local backbackground = display.newImage("background.png")
backbackground.x = 240
backbackground.y = 160

local backgroundfar = display.newImage("bgfar1.png")
backgroundfar.x = 480
backgroundfar.y = 160

local backgroundnear1 = display.newImage("bgnear2.png")
backgroundnear1.x = 240
backgroundnear1.y = 160

local backgroundnear2 = display.newImage("bgnear2.png")
backgroundnear2.x = 760
backgroundnear2.y = 160

local blocks = display.newGroup()
local player = display.newGroup()
local screen = display.newGroup()
local ghosts = display.newGroup()
local spikes = display.newGroup()
local blasts = display.newGroup()

local groundMin = 420
local groundMax = 340
local groundLevel = groundMin
local speed = 5

--create ghosts
for a = 1, 3, 1 do
	ghost = display.newImage("ghost.png")
	ghost.name = ("ghost" .. a)
	ghost.id = a
	ghost.x = 800
	ghost.y = 600
	ghost.speed = 0;
	ghost.isAlive = false
	ghost.alpha = .5
	ghosts:insert(ghost)
end

--create spikes
for a = 1, 3, 1 do
	spike = display.newImage("spikeBlock.png")
	spike.name = ("spike" .. a)
	spike.id = a
	spike.x = 900
	spike.y = 500
	spike.isAlive = false
	spikes:insert(spike)
end

--create blasts
for a=1, 5, 1 do
	blast = display.newImage("blast.png")
	blast.name = ("blast" .. a)
	blast.id = a
	blast.x = 800
	blast.y = 500
	blast.isAlive = false
	blasts:insert(blast)
end

for a = 1, 8, 1 do
	isDone = false
	numGen = math.random(2)
	local newBlock
	print (numGen)
	if(numGen == 1 and isDone == false) then
		newBlock = display.newImage("ground1.png")
		isDone = true
	end
	
	if(numGen == 2 and isDone == false) then
		newBlock = display.newImage("ground2.png")
		isDone = true
	end
	
	if(numGen == 3 and isDone == false) then
		newBlock = display.newImage("ground3.png")
		isDone = true
	end
	
	if(isDone == false) then
		newBlock = display.newImage("ground1.png")
	end
	newBlock.name = ("block" .. a)
	newBlock.id = a
	newBlock.x = (a * 79) - 79
	newBlock.y = groundLevel
	blocks:insert(newBlock)
end

--create our sprite sheet
local spriteSheet = sprite.newSpriteSheet("monsterSpriteSheet.png", 100, 100)
local monsterSet = sprite.newSpriteSet(spriteSheet, 1, 7)
sprite.add(monsterSet, "running", 1, 6, 600, 0)
sprite.add(monsterSet, "jumping", 7, 7, 1, 1)

--set the different variables we will use for our monster sprite
--also sets and starts the first animation for the monster
local monster = sprite.newSprite(monsterSet)
monster:prepare("running")
monster:play()
monster.x = 110
monster.y = 200
--these are 2 variables that will control the falling and jumping of the monster
monster.gravity = -6
monster.accel = 0

--rectangle used for our collision detection
--it will always be in front of the monster sprite
--that way we know if the monster hit into anything
local collisionRect = display.newRect(monster.x + 36, monster.y, 1, 70)
collisionRect.strokeWidth = 1
collisionRect:setFillColor(140, 140, 140)
collisionRect:setStrokeColor(180, 180, 180)
collisionRect.alpha = 0

--used to put everything on the screen into the screen group
--this will let us change the order in which sprites appear on
--the screen if we want. The earlier it is put into the group the
--further back it will go
screen:insert(backbackground)
screen:insert(backgroundfar)
screen:insert(backgroundnear1)
screen:insert(backgroundnear2)
screen:insert(blocks)
screen:insert(spikes)
screen:insert(blasts)
screen:insert(ghosts)
screen:insert(monster)
screen:insert(collisionRect)

--the update function will control most everything that happens in our game
--this will be called every frame(30 frames per second in our case(the default in corona))
local function update( event )
	updateBackgrounds()
	updateSpeed()
	updateMonster()
	updateBlocks()
	checkCollisions()
	updateBlasts()
	updateSpikes()
	updateGhosts()
end

function checkCollisions()
        --boolean variable so we know if we were on the ground in the last frame
	wasOnGround = onGround

	--checks to see if the collisionRect has collided with anything. This is why it is lifted off of the ground
	--a little bit, if it hits something we will want our monster to do something... like die! This is why we don't want it
        --hitting the ground, it wouldn't make sense for the monster to die everything he touched the ground. We check this by cycling through
	--all of the ground pieces in the blocks group and comparing their x and y coordinates to that of the collisionRect
	for a = 1, blocks.numChildren, 1 do
		if(collisionRect.y - 10> blocks[a].y - 170 and blocks[a].x - 40 < collisionRect.x and blocks[a].x + 40 > collisionRect.x) then
			--stop the monster
			speed = 0
		end
	end
	
	for a = 1, spikes.numChildren, 1 do
		if(spikes[a].isAlive == true) then
			if(collisionRect.y - 10> spikes[a].y - 170 and spikes[a].x - 40 < collisionRect.x and spikes[a].x + 40 > collisionRect.x) then
				--stop the monster
				speed = 0
			end
		end
	end
	
	for a = 1, ghosts.numChildren, 1 do
		if(ghosts[a].isAlive == true) then
			--print(monster.y .. "::" .. ghosts[a].y .. "::" .. ((monster.y - ghosts[a].y))/2 )
			if(((  ((monster.y-ghosts[a].y))<70) and ((monster.y - ghosts[a].y) > -70)) and (ghosts[a].x - 40 < collisionRect.x and ghosts[a].x + 40 > collisionRect.x)) then
				--stop the monster
				speed = 0
			end
		end
	end

	--this is where we check to see if the monster is on the ground or in the air, if he is in the air then he can't jump(sorry no double
	--jumping for our little monster, however if you did want him to be able to double jump like Mario then you would just need
	--to make a small adjustment here, by adding a second variable called something like hasJumped. Set it to false normally, and turn it to
	--true once the double jump has been made. That way he is limited to 2 hops per jump.
	--Again we cycle through the blocks group and compare the x and y values of each.
	for a = 1, blocks.numChildren, 1 do
		if(monster.y >= blocks[a].y - 170 and blocks[a].x < monster.x + 60 and blocks[a].x > monster.x - 60) then
			monster.y = blocks[a].y - 171
			onGround = true
			break
		else
			onGround = false
		end
	end
end

function updateBlasts()
	for a = 1, blasts.numChildren, 1 do
		if(blasts[a].isAlive == true) then
			(blasts[a]):translate(5, 0)
			
			if(blasts[a].x > 550) then
				blasts[a].x = 800
				blasts[a].y = 500				
				blasts[a].isAlive = false;
			end
		end
	
		for b = 1, spikes.numChildren, 1 do
			if(spikes[b].isAlive == true) then	
				--print(blasts[a].x .. "::" .. blasts[a].y .. "::" .. spikes[b].x .. "::" .. spikes[b].y)
				--             133  >  140
				if(blasts[a].y - 25 > spikes[b].y - 120 and blasts[a].y + 25 < spikes[b].y + 120 and spikes[b].x - 40 < blasts[a].x + 25 and spikes[b].x + 40 > blasts[a].x - 25) then
					blasts[a].x = 800
					blasts[a].y = 500					
					blasts[a].isAlive = false;
					spikes[b].x = 900
					spikes[b].y = 500
					spikes[b].isAlive = false;					
				end
			end
		end
		
		for b = 1, ghosts.numChildren, 1 do
			if(ghosts[b].isAlive == true) then	
				--print(blasts[a].x .. "::" .. blasts[a].y .. "::" .. spikes[b].x .. "::" .. spikes[b].y)
				--             133  >  140
				if(blasts[a].y - 25 > ghosts[b].y - 120 and blasts[a].y + 25 < ghosts[b].y + 120 and ghosts[b].x - 40 < blasts[a].x + 25 and ghosts[b].x + 40 > blasts[a].x - 25) then
					blasts[a].x = 800
					blasts[a].y = 500					
					blasts[a].isAlive = false;
					ghosts[b].x = 800
					ghosts[b].y = 600
					ghosts[b].isAlive = false;	
					ghosts[b].speed = 0
				end
			end
		end
		
	end
end

function updateMonster()
	--if our monster is jumping then switch to the jumping animation
	--if not keep playing the running animation
	if(onGround) then
		if(wasOnGround) then

		else
			monster:prepare("running")
			monster:play()
		end
	else
		monster:prepare("jumping")
		monster:play()
	end

	if(monster.accel > 0) then
		monster.accel = monster.accel - 1
	end

	--update the monsters position, accel is used for our jump and
	--gravity keeps the monster coming down. You can play with those 2 variables
	--to make lots of interesting combinations of gameplay like 'low gravity' situations
	monster.y = monster.y - monster.accel
	monster.y = monster.y - monster.gravity

	--update the collisionRect to stay in front of the monster
	collisionRect.y = monster.y
end

--this is the function that handles the jump events. If the screen is touched on the left side
--then make the monster jump
function touched( event )
	if(event.phase == "began") then
		if(event.x < 241) then
			if(onGround) then
				monster.accel = monster.accel + 20
			end
		else
			for a=1, blasts.numChildren, 1 do
				if(blasts[a].isAlive == false) then
					blasts[a].isAlive = true;
					blasts[a].x = monster.x + 50
					blasts[a].y = monster.y
					break
				end
			end
		end		
	end
end

function updateSpeed()
	speed = speed + .0005
end

function updateBlocks()
     for a = 1, blocks.numChildren, 1 do
          if(a > 1) then
               newX = (blocks[a - 1]).x + 79
          else
               newX = (blocks[8]).x + 79 - speed
          end
          if((blocks[a]).x < -40) then
			 if(inEvent == 11) then
				  (blocks[a]).x, (blocks[a]).y = newX, 600
			 else
				  (blocks[a]).x, (blocks[a]).y = newX, groundLevel
			 end
			 if(inEvent == 12) then
				for a=1, spikes.numChildren, 1 do
					if(spikes[a].isAlive == true) then
						--do nothing
					else
						spikes[a].isAlive = true
						spikes[a].y = groundLevel - 200
						spikes[a].x = newX
						break
					end
				end
			end
			 checkEvent()
			else
				 (blocks[a]):translate(speed * -1, 0)
			end
		 end
end

function updateSpikes()
	for a = 1, spikes.numChildren, 1 do
		if(spikes[a].isAlive == true) then
			(spikes[a]):translate(speed * -1, 0)
			
			if(spikes[a].x < -80) then
				spikes[a].x = 900
				spikes[a].y = 500
				spikes[a].isAlive = false;
			end
		end		
	end
end

function updateGhosts()
	for a = 1, ghosts.numChildren, 1 do
		if(ghosts[a].isAlive == true) then
			(ghosts[a]):translate(speed * -1, 0)
			
			if(ghosts[a].y > monster.y) then
				ghosts[a].y = ghosts[a].y - 1			
			end
			
			if(ghosts[a].y < monster.y) then
				ghosts[a].y = ghosts[a].y + 1			
			end
			
			if(ghosts[a].x < -80) then
				ghosts[a].x = 800
				ghosts[a].y = 600
				ghosts[a].speed = 0
				ghosts[a].isAlive = false;
			end
		end		
	end
end

function checkEvent()
     --first check to see if we are already in an event, we only want 1 event going on at a time
     if(eventRun > 0) then
          --if we are in an event decrease eventRun. eventRun is a variable that tells us how
          --much longer the event is going to take place. Everytime we check we need to decrement
          --it. Then if at this point eventRun is 0 then the event has ended so we set inEvent back
          --to 0.
          eventRun = eventRun - 1
          if(eventRun == 0) then
               inEvent = 0
          end
     end
     --if we are in an event then do nothing
     if(inEvent > 0 and eventRun > 0) then
          --Do nothing
     else
          --if we are not in an event check to see if we are going to start a new event. To do this
          --we generate a random number between 1 and 100. We then check to see if our 'check' is
          --going to start an event. We are using 100 here in the example because it is easy to determine
          --the likelihood that an event will fire(We could just as easilt chosen 10 or 1000).
          --For example, if we decide that an event is going to
          --start everytime check is over 80 then we know that everytime a block is reset there is a 20%
          --chance that an event will start. So one in every five blocks should start a new event. This
          --is where you will have to fit the needs of your game.
          check = math.random(100)
 
          --this first event is going to cause the elevation of the ground to change. For this game we
          --only want the elevation to change 1 block at a time so we don't get long runs of changing
          --elevation that is impossible to pass so we set eventRun to 1.
          if(check > 80 and check < 99) then
               --since we are in an event we need to decide what we want to do. By making inEvent another
               --random number we can now randomly choose which direction we want the elevation to change.
               inEvent = math.random(10)
               eventRun = 1
          end		  
		  if(check > 98) then
				 inEvent = 11
				 eventRun = 2
			end
		if(check > 72 and check < 81) then
				inEvent = 12
				eventRun = 1
		end
		if(check > 60 and check < 73) then
				inEvent = 13
				eventRun = 1
		end	
     end
     --if we are in an event call runEvent to figure out if anything special needs to be done
     if(inEvent > 0) then
          runEvent()
     end
end
--this function is pretty simple it just checks to see what event should be happening, then
--updates the appropriate items. Notice that we check to make sure the ground is within a
--certain range, we don't want the ground to spawn above or below whats visible on the screen.
function runEvent()
     if(inEvent < 6) then
          groundLevel = groundLevel + 40
     end
     if(inEvent > 5 and inEvent < 11) then
          groundLevel = groundLevel - 40
     end
     if(groundLevel < groundMax) then
          groundLevel = groundMax
     end
     if(groundLevel > groundMin) then
          groundLevel = groundMin
     end
	 if(inEvent == 13) then	 
		for a=1, ghosts.numChildren, 1 do
			if(ghosts[a].isAlive == false) then
				ghosts[a].isAlive = true;
				ghosts[a].x = 500
				ghosts[a].y = math.random(-50, 400)
				ghosts[a].speed = math.random(2,4)
				break
			end
		end
	 end
end

function updateBackgrounds()
	--far background movement
	backgroundfar.x = backgroundfar.x - (speed/55)
	
	--near background movement
	backgroundnear1.x = backgroundnear1.x - (speed/5)
	if(backgroundnear1.x < -239) then
		backgroundnear1.x = 760
	end
	
	backgroundnear2.x = backgroundnear2.x - (speed/5)
	if(backgroundnear2.x < -239) then
		backgroundnear2.x = 760
	end
end

--this is how we call the update function, make sure that this line comes after the actual function or it will not be able to find it
--timer.performWithDelay(how often it will run in milliseconds, function to call, how many times to call(-1 means forever))
timer.performWithDelay(1, update, -1)
Runtime:addEventListener("touch", touched, -1)